#!/usr/bin/python3
import os
import sys
import json
from configparser import ConfigParser
import traceback
import uuid
import datetime
import logging
from logging.handlers import RotatingFileHandler

from flask import Flask
from flask import jsonify, request, Flask, render_template, redirect, url_for

app = Flask("app")

gMonitorLog = None
logging.basicConfig()
Rthandler = RotatingFileHandler('/var/log/cockpit-find-cli.log' ,
        maxBytes=2 * 1024 * 1024, backupCount=2)
formatter = logging.Formatter('%(asctime)s %(filename)s[line:%(lineno)d] %(levelname)s %(message)s')
Rthandler.setFormatter(formatter)

gMonitorLog = logging.getLogger('gMonitorLog')
gMonitorLog.setLevel(logging.INFO)
gMonitorLog.addHandler(Rthandler)

gServerPort = 9092

def wrapInfo(msg):
    dt_prefix = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S: ")
    gMonitorLog.info(dt_prefix + msg)

def wrapError(msg):
    dt_prefix = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S: ")
    gMonitorLog.error(dt_prefix + msg)

# cockpif find user name
gUsername = "cockpit-client"

def createUser():
    global gUsername

    userexists = False
    with open("/etc/passwd", "r") as fd:
        line = fd.readline()
        while line:
            if line.find(gUsername) >= 0:
                userexists = True
                break
            line = fd.readline()

    if not userexists:
        ret = os.system("useradd --system -d /etc/%s %s" %(gUsername, gUsername))
        if not ret>>8 == 0:
            wrapError("create user %s failed" %gUsername)
            sys.exit(1)

    if not os.path.exists("/etc/%s/.ssh" %gUsername):
        cmd = "mkdir -p /etc/%s/.ssh" %gUsername
        ret = os.system(cmd)
        if not ret>>8 == 0:
            wrapError("exec cmd failed: %s" %cmd)
            sys.exit(1)

    cmd = "cat /etc/sudoers | grep '%s ALL=(ALL) NOPASSWD: ALL' >/dev/null || (chmod a+w /etc/sudoers; echo '%s ALL=(ALL) NOPASSWD: ALL' >> /etc/sudoers; chmod a-w /etc/sudoers)" %(gUsername, gUsername)
    ret = os.system(cmd)
    if not ret>>8 == 0:
        wrapError("exec cmd failed: %s" %cmd)
        sys.exit(1)

    # 修改 sshd 的配置，不使用 dns，使用 dns 会导致很慢
    cmd = "cat /etc/ssh/sshd_config | grep '# scmms change flag' >/dev/null || (sed 's/UseDNS/#UseDNS/g' -i /etc/ssh/sshd_config && echo 'UseDNS no' >> /etc/ssh/sshd_config && systemctl restart sshd && echo '# scmms change flag' >> /etc/ssh/sshd_config)"
    ret = os.system(cmd)
    if not ret>>8 == 0:
        wrapError("exec cmd failed: %s" %cmd)
        sys.exit(1)

    wrapInfo("create user %s succeed" %gUsername)

createUser()

@app.route("/")
def index():
    return "hello world..."

@app.route("/uploadCredit", methods=["POST"])
def uploadCredit():
    global gUsername

    args = json.loads(request.get_data())
    key = args.get("key")
    authstr = args.get("auth")
    authstr = authstr.strip()
    if not key == "qazwsxedcrfv":
        return json.dumps({"succeed": False, "error": "key invalid"})

    if not authstr:
        return json.dumps({"succeed": False, "error": "auth invalid"})

    try:
        cmd = "cat /etc/%s/.ssh/authorized_keys | grep '" %gUsername + authstr + "' >/dev/null"
        ret = os.system(cmd)
        if not ret>>8 == 0:
            cmd = "echo " + authstr + " >>/etc/%s/.ssh/authorized_keys" %gUsername
            ret = os.system(cmd)
            if not ret>>8 == 0:
                return json.dumps({"succeed": False, "error": "upload auth string to file failed"})

        # 返回内容中带上版本等信息，js 前端就可以不用调用命令获取了
        data = {}
        with open("/etc/machine-id", "r") as fd:
            data["machineid"] = fd.readlines()

        bexist = os.path.exists('/etc/hostname')
        if bexist == False:
            r = os.popen("hostname")
            data["hostname"] = r.readlines()
            r.close()
        else:
            with open("/etc/hostname", "r") as fd:
                data["hostname"] = fd.readlines()

        with open("/etc/.kyinfo", "r") as fd:
            data["kyinfo"] = fd.readlines()

        return json.dumps({"succeed": True, "error": "", "data": data})
    except:
        wrapError("setAssert failed")
        wrapError(traceback.format_exc())
        return json.dumps({"succeed": False, "error": "数据库操作失败"})

if __name__ == "__main__":
    pass
    # flask 自带的服务器可以用于开发、调试，用于产品中的需要更专业的服务器软件 gunicorn
    #gunicorn -w 4 -b 0.0.0.0:9092 flaskserver:app
    #app.run(debug=True, host="0.0.0.0", port=gServerPort)
